@@ -2,10 +2,10 @@
  * 
  * hlist.c -
  * 
- * $Id: hlist.c,v 1.51.4.20 2006/09/23 08:26:44 opengl2772 Exp $
+ * $Id: hlist.c,v 1.51.4.22 2008/03/06 15:38:08 opengl2772 Exp $
  * 
  * Copyright (C) 1997-1999 Satoru Takabayashi All rights reserved.
- * Copyright (C) 2000-2006 Namazu Project All rights reserved.
+ * Copyright (C) 2000-2008 Namazu Project All rights reserved.
  * This is free software with ABSOLUTELY NO WARRANTY.
  * 
  * This program is free software; you can redistribute it and/or modify
@@ -26,10 +26,7 @@
  * This file must be encoded in EUC-JP encoding
  * 
  */
-#include <stdio.h>
-#include <stdlib.h>
-#include <math.h>
-#include <errno.h>
+
 #ifdef HAVE_CONFIG_H
 #  include "config.h"
 #endif
@@ -37,6 +34,16 @@
 #  include "support.h"
 #endif
 
+#include <stdio.h>
+#ifdef HAVE_STDLIB_H
+#  include <stdlib.h>
+#endif
+#include <math.h>
+
+#ifdef HAVE_ERRNO_H
+#  include <errno.h>
+#endif
+
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>
 #else
@@ -46,9 +53,9 @@
 #endif
 
 #ifdef HAVE_STRING_H
-#include <string.h>
+#  include <string.h>
 #else
-#include <strings.h>
+#  include <strings.h>
 #endif
 
 #include "libnamazu.h"
@@ -60,6 +67,7 @@
 #include "search.h"
 #include "query.h"
 #include "score.h"
+
 static int document_number = 0;  /* Number of documents covered in a target index */
 static char field_for_sort[BUFSIZE] = "";  /* field_for_sort name used with sorting */
 
@@ -102,7 +110,8 @@ field_sort(NmzResult hlist)
 
     for (i = 0; i < hlist.num; i++) {
 	char buf[BUFSIZE];
-	int leng;
+	size_t leng;
+
 	nmz_get_field_data(hlist.data[i].idxid, 
 			   hlist.data[i].docid, field_for_sort, buf);
 	nmz_chomp(buf);
