@@ -2,10 +2,10 @@
  * 
  * idxname.c - Idx handling routines.
  *
- * $Id: idxname.c,v 1.25.8.1 2001/09/02 07:13:37 rug Exp $
+ * $Id: idxname.c,v 1.25.8.9 2008/03/06 15:38:09 opengl2772 Exp $
  * 
  * Copyright (C) 1997-1999 Satoru Takabayashi All rights reserved.
- * Copyright (C) 2000 Namazu Project All rights reserved.
+ * Copyright (C) 2000-2007 Namazu Project All rights reserved.
  * Copyright (C) 1999 NOKUBI Takatsugu All rights reserved.
  * This is free software with ABSOLUTELY NO WARRANTY.
  * 
@@ -27,9 +27,6 @@
  * 
  */
 
-#include <errno.h>
-#include <stdlib.h>
-#include <ctype.h>
 #ifdef HAVE_CONFIG_H
 #  include "config.h"
 #endif
@@ -37,10 +34,19 @@
 #  include "support.h"
 #endif
 
+#ifdef HAVE_ERRNO_H
+#  include <errno.h>
+#endif
+
+#ifdef HAVE_STDLIB_H
+#  include <stdlib.h>
+#endif
+#include <ctype.h>
+
 #ifdef HAVE_STRING_H
-#include <string.h>
+#  include <string.h>
 #else
-#include <strings.h>
+#  include <strings.h>
 #endif
 
 #include "libnamazu.h"
@@ -153,7 +159,8 @@ nmz_complete_idxnames(void)
     int i;
 
     for (i = 0; i < indices.num; i++) {
- 	if (*indices.names[i] == '+' && isalnum(*(indices.names[i] + 1))) {
+ 	if (*indices.names[i] == '+' && 
+	    nmz_isalnum((unsigned char)*(indices.names[i] + 1))) {
 	    char *tmp;
 	    tmp = malloc(strlen(defaultidx) 
 				  + 1 + strlen(indices.names[i]) + 1);
@@ -262,7 +269,8 @@ nmz_push_hitnum(struct nmz_hitnumlist *hn,
 void 
 nmz_set_defaultidx(const char *idx)
 {
-    strcpy(defaultidx, idx);
+    strncpy(defaultidx, idx, BUFSIZE - 1);
+    defaultidx[BUFSIZE - 1] = '\0';
 }
 
 char *
