@@ -1,7 +1,7 @@
 # -*- Perl -*-
-# $Id: var.pl.in,v 1.12.8.1 2003/09/23 07:32:00 knok Exp $
+# $Id: var.pl.in,v 1.12.8.3 2006/01/24 12:42:55 opengl2772 Exp $
 # Copyright (C) 1997-1999 Satoru Takabayashi All rights reserved.
-# Copyright (C) 2000 Namazu Project All rights reserved.
+# Copyright (C) 2000-2006 Namazu Project All rights reserved.
 #     This is free software with ABSOLUTELY NO WARRANTY.
 #
 #  This program is free software; you can redistribute it and/or modify
@@ -29,9 +29,10 @@ package var;
 # Software information.
 #
 
-$VERSION         = "2.0.13";
-$COPYRIGHT       = "Copyright (C) 1997-1999 Satoru Takabayashi All rights reserved.\nCopyright (C) 2000-2004 Namazu Project All rights reserved."; # should be doublequote
+$VERSION         = "2.0.18";
+$COPYRIGHT       = "Copyright (C) 1997-1999 Satoru Takabayashi All rights reserved.\nCopyright (C) 2000-2008 Namazu Project All rights reserved."; # should be doublequote
 $MAILING_ADDRESS = 'bug-namazu@namazu.org';
+$TRAC_URI        = 'http://www.namazu.org/trac-namazu/trac.cgi';
 
 #-----------------------------------------------------------
 #
@@ -108,6 +109,8 @@ $Opt{'noupdate'} = 0;
 
 $Opt{'checkfilesize'} = 0;
 
+$Opt{'decodebase64'} = 0;
+
 #-----------------------------------------------------------
 #
 # Size of `int'
