@@ -2,10 +2,10 @@
  * 
  * cgi.c -
  * 
- * $Id: cgi.c,v 1.61.4.15 2004/03/20 15:02:51 usu Exp $
+ * $Id: cgi.c,v 1.61.4.22 2007/12/05 16:59:35 opengl2772 Exp $
  * 
  * Copyright (C) 1997-1999 Satoru Takabayashi All rights reserved.
- * Copyright (C) 2000-2004 Namazu Project All rights reserved.
+ * Copyright (C) 2000-2007 Namazu Project All rights reserved.
  * This is free software with ABSOLUTELY NO WARRANTY.
  * 
  * This program is free software; you can redistribute it and/or modify
@@ -25,13 +25,15 @@
  * 
  * 
  */
+
+#ifdef HAVE_CONFIG_H
+#  include "config.h"
+#endif
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <ctype.h>
 #include <assert.h>
-#ifdef HAVE_CONFIG_H
-#  include "config.h"
-#endif
 
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>
@@ -42,9 +44,9 @@
 #endif
 
 #ifdef HAVE_STRING_H
-#include <string.h>
+#  include <string.h>
 #else
-#include <strings.h>
+#  include <strings.h>
 #endif
 
 #include "namazu.h"
@@ -121,11 +123,14 @@ validate_idxname(const char * idxname)
 #endif
 
     if (*idxname == '\0' || *idxname == '/' || (win32 && *idxname == '\\')) {
-        printf("%s text/html" CRLF CRLF, MSG_MIME_HEADER);
+        printf("%s %s; %s" CRLF CRLF, MSG_MIME_HEADER,
+            "text/html", _("charset=ISO-8859-1"));
+        printf("<html><body>\n");
 	puts_entitize(idxname);
 	printf(" : ");
         printf(_("Invalid idxname."));
-        exit(EXIT_FAILURE);
+        printf("\n</body></html>\n");
+        exit_nmz(EXIT_FAILURE);
     }
     while (*idxname) {
         if (nmz_strprefixcasecmp("../", idxname) == 0 ||
@@ -133,11 +138,14 @@ validate_idxname(const char * idxname)
             (win32 && nmz_strprefixcasecmp("..\\", idxname) == 0) || 
             (win32 && nmz_strprefixcasecmp("..." , idxname) == 0)) 
         {
-	    printf("%s text/html" CRLF CRLF, MSG_MIME_HEADER);
+            printf("%s %s; %s" CRLF CRLF, MSG_MIME_HEADER,
+                "text/html", _("charset=ISO-8859-1"));
+            printf("<html><body>\n");
 	    puts_entitize(idxname);
 	    printf(" : ");
             printf(_("Invalid idxname."));
-            exit(EXIT_FAILURE);
+            printf("\n</body></html>\n");
+            exit_nmz(EXIT_FAILURE);
         }
 	/* Skip until next '/' */
 	while (*idxname && *idxname != '/' && !(win32 && *idxname == '\\')) {
@@ -251,6 +259,7 @@ get_cgi_vars(const char *querystring)
     while (1) {
 	int len;
 	const char *tmp;
+        char *pSemicolon;
 	char name[BUFSIZE];
 	char value[BUFSIZE];
 
@@ -271,7 +280,11 @@ get_cgi_vars(const char *querystring)
 
 	querystring++;
 	tmp = strchr(querystring, '&');
-	if (tmp == NULL) {
+        pSemicolon = strchr(querystring, ';');
+        if (tmp == NULL || (pSemicolon && (tmp > pSemicolon))) {
+            tmp = pSemicolon;
+        }
+        if (tmp == NULL) {
 	    tmp = querystring + strlen(querystring);  /* last point: '\0' */
 	}
 	len = tmp - querystring;
@@ -288,7 +301,7 @@ get_cgi_vars(const char *querystring)
 
 	if (cv == NULL) {
 	    fprintf(stderr, "an error occurred at add_cgivar.\n");
-	    exit(EXIT_FAILURE);
+	    exit_nmz(EXIT_FAILURE);
 	}
 
 	if (*querystring == '\0') {
@@ -433,7 +446,9 @@ process_cgi_var_max(char *value, struct cgiarg *ca)
 {
     int max;
 
-    sscanf(value, "%d", &max);
+    if (sscanf(value, "%d", &max) != 1) {
+        return;
+    }
 
     if (max < 0)
 	max = 0;
@@ -448,7 +463,10 @@ process_cgi_var_whence(char *value, struct cgiarg *ca)
 {
     int whence;
 
-    sscanf(value, "%d", &whence);
+    if (sscanf(value, "%d", &whence) != 1) {
+        return;
+    }
+
     if (whence < 0) {
 	whence = 0;
     }
@@ -532,7 +550,7 @@ init_cgi(char *query, char *subquery)
 
     if (process_cgi_vars(&ca)) {
 	show_mini_usage();   /* if it is NOT CGI, show usage and exit */
-	exit(EXIT_FAILURE);
+	exit_nmz(EXIT_FAILURE);
     }
 
     if (nmz_get_idxnum() == 0) {
