@@ -1,9 +1,9 @@
 /*
  * 
- * $Id: rcfile.c,v 1.31.4.3 2001/12/21 03:30:24 knok Exp $
+ * $Id: rcfile.c,v 1.31.4.16 2007/12/05 16:59:36 opengl2772 Exp $
  * 
  * Copyright (C) 1997-1999 Satoru Takabayashi All rights reserved.
- * Copyright (C) 2000 Namazu Project All rights reserved.
+ * Copyright (C) 2000-2007 Namazu Project All rights reserved.
  * This is free software with ABSOLUTELY NO WARRANTY.
  * 
  * This program is free software; you can redistribute it and/or modify
@@ -23,10 +23,6 @@
  * 
  */
 
-#include <assert.h>
-#include <stdio.h>
-#include <stdlib.h>
-#include <errno.h>
 #ifdef HAVE_CONFIG_H
 #  include "config.h"
 #endif
@@ -34,10 +30,18 @@
 #  include "support.h"
 #endif
 
+#include <assert.h>
+#include <stdio.h>
+#include <stdlib.h>
+
+#ifdef HAVE_ERRNO_H
+#  include <errno.h>
+#endif
+
 #ifdef HAVE_STRING_H
-#include <string.h>
+#  include <string.h>
 #else
-#include <strings.h>
+#  include <strings.h>
 #endif
 
 #include "namazu.h"
@@ -55,6 +59,13 @@
 #include "idxname.h"
 #include "output.h"
 #include "score.h"
+#include "charset.h"
+
+/*
+ * Extern variables.
+ */
+
+extern NMZ_HANDLE handle_charset;
 
 /*
  * Default directory to place namazurc.
@@ -66,8 +77,13 @@ static char *namazurcdir = OPT_CONFDIR;
  */
 static char user_namazurc[BUFSIZE] = "";
 
+static char namazunorc[BUFSIZE] = "";
+
 static char *errmsg  = NULL;
 
+/* suicide time (sec) */
+int suicide_time = SUICIDE_TIME;
+
 /* 
  * Storing loaded rcfile names for show_config().
  *
@@ -130,6 +146,9 @@ static enum nmz_stat process_rc_template ( const char *directive, const StrList
 static enum nmz_stat process_rc_maxhit ( const char *directive, const StrList *args );
 static enum nmz_stat process_rc_maxmatch ( const char *directive, const StrList *args );
 static enum nmz_stat process_rc_contenttype ( const char *directive, const StrList *args );
+static enum nmz_stat process_rc_suicidetime ( const char *directive, const StrList *args );
+static enum nmz_stat process_rc_regexsearch ( const char *directive, const StrList *args );
+static enum nmz_stat process_rc_charset ( const char *directive, const StrList *args );
 
 struct conf_directive {
     char *name;
@@ -156,6 +175,9 @@ static struct conf_directive directive_tab[] = {
     { "MAXHIT",        1, 0, process_rc_maxhit },
     { "MAXMATCH",      1, 0, process_rc_maxmatch },
     { "CONTENTTYPE",   1, 0, process_rc_contenttype },
+    { "SUICIDE_TIME",  1, 0, process_rc_suicidetime },
+    { "REGEX_SEARCH",  1, 0, process_rc_regexsearch },
+    { "CHARSET",       2, 0, process_rc_charset },
     { NULL,            0, 0, NULL }
 };
 
@@ -335,6 +357,41 @@ process_rc_contenttype(const char *directive, const StrList *args)
     return SUCCESS;
 }
 
+static enum nmz_stat
+process_rc_suicidetime(const char *directive, const StrList *args)
+{
+    int arg1 = atoi(args->value);
+
+    suicide_time = arg1;
+    return SUCCESS;
+}
+
+static enum nmz_stat
+process_rc_regexsearch(const char *directive, const StrList *args)
+{
+    char *arg1 = args->value;
+
+    if (strcasecmp(arg1, "ON") == 0) {
+	nmz_set_regex_searchmode(1);
+    } else if (strcasecmp(arg1, "OFF") == 0) {
+	nmz_set_regex_searchmode(0);
+    }
+
+    return SUCCESS;
+}
+
+static enum nmz_stat
+process_rc_charset(const char *directive, const StrList *args)
+{
+    char *arg1 = args->value;
+    char *arg2 = args->next->value;
+
+    if (nmz_add_strlist(handle_charset, arg1, arg2) != SUCCESS) {
+        return FAILURE;
+    }
+    return SUCCESS;
+}
+
 /* 
  * Get the environment variable of NAMAZURC, NAMAZUCONF or
  * NAMAZUCONFPATH.  and return it. Original of this code is
@@ -368,9 +425,10 @@ getenv_namazurc(void)
 static int 
 get_rc_arg(const char *line, char *arg)
 {
+    arg[BUFSIZE - 1] = '\0';
     *arg = '\0';
     if (*line != '"') {
-	int n = strcspn(line, " \t");
+	size_t n = strcspn(line, " \t");
 	if (n >= BUFSIZE) n = BUFSIZE - 1;
 	strncpy(arg, line, n);
 	arg[n] = '\0';     /* Hey, don't forget this after strncpy()! */
@@ -380,7 +438,7 @@ get_rc_arg(const char *line, char *arg)
 	line++;
 	n = 1;
 	do {
-	    int nn = strcspn(line, "\"\\");
+	    size_t nn = strcspn(line, "\"\\");
 	    if (nn >= (BUFSIZE - strlen(arg))) nn = BUFSIZE - strlen(arg) - 1;
 	    strncat(arg, line, nn);
 	    n += nn;
@@ -408,7 +466,11 @@ get_rc_arg(const char *line, char *arg)
 static void 
 replace_home(char *str)
 {
-    char tmp[BUFSIZE];
+    char tmp[BUFSIZE] = "";
+
+    if (str == NULL || *str == '\0') {
+        return;
+    }
 
     strcpy(tmp, str);
     if (nmz_strprefixcmp(tmp, "~/") == 0) {
@@ -441,7 +503,8 @@ add_strlist(StrList *list, const char *arg)
 
     newp->value = malloc(strlen(arg) + 1);
     if (newp->value == NULL) {
-	 return NULL;
+        free(newp);
+	return NULL;
     }
     strcpy(newp->value, arg);
     newp->next = NULL;
@@ -501,7 +564,7 @@ get_rc_args(const char *line)
 
     /* Get a directive name. */
     {
-	char directive[BUFSIZE];
+	char directive[BUFSIZE] = "";
 	n = strspn(line, DIRECTIVE_CHARS);
 	if (n == 0) {
 	    errmsg = _("invalid directive name");
@@ -523,7 +586,7 @@ get_rc_args(const char *line)
     }
 
     while (1) {
-	char arg[BUFSIZE];
+	char arg[BUFSIZE] = "";
 	n = get_rc_arg(line, arg);
 	if (n == 0) { /* cannot get arg1 */
 	    return NULL;
@@ -654,7 +717,7 @@ static enum nmz_stat
 load_rcfile(const char *fname)
 {
     FILE *fp;
-    char buf[BUFSIZE];
+    char buf[BUFSIZE] = "";
     int lineno = 1;
 
     fp = fopen(fname, "rb");
@@ -663,7 +726,7 @@ load_rcfile(const char *fname)
 	return FAILURE;
     }
 
-    while (fgets(buf, BUFSIZE - 1, fp) != NULL) {
+    while (fgets(buf, BUFSIZE, fp) != NULL) {
 	int current_lineno = lineno;
 	
 	do {
@@ -674,6 +737,10 @@ load_rcfile(const char *fname)
 		
 		buf[strlen(buf) - 1] = '\0'; /* Remove ending \ */
 		remaining = BUFSIZE - strlen(buf) - 1;
+		if (remaining == 0) {
+		    nmz_chomp(buf);
+		    break;
+		}
 		if (fgets(buf + strlen(buf), remaining, fp) == NULL) {
 		    nmz_chomp(buf);
 		    break;
@@ -687,6 +754,7 @@ load_rcfile(const char *fname)
 	if (parse_rcfile(buf, current_lineno) != SUCCESS) {
 	    nmz_set_dyingmsg(nmz_msg(_("%s:%d: syntax error: %s"),  
 				     fname, current_lineno, errmsg));
+            fclose(fp);
 	    return FAILURE;
 	}
     }
@@ -708,7 +776,16 @@ load_rcfile(const char *fname)
 void
 set_namazurc(const char *arg)
 {
-    strcpy(user_namazurc, arg);
+    strncpy(user_namazurc, arg, BUFSIZE - 1);
+    user_namazurc[BUFSIZE - 1] = '\0';
+}
+
+void
+set_namazunorc(const char *arg)
+{
+    strncpy(namazunorc, arg, BUFSIZE - 1);
+    namazunorc[BUFSIZE - 1] = '\0';
+    nmz_strlower(namazunorc);
 }
 
 /*
@@ -726,18 +803,33 @@ set_namazurc(const char *arg)
 enum nmz_stat 
 load_rcfiles(void)
 {
+    char *env_norc, _norc[BUFSIZE] = "";
+
+    if ((env_norc = getenv("NAMAZUNORC"))) {
+        strncpy(_norc, env_norc, BUFSIZE - 1);
+        _norc[BUFSIZE - 1] = '\0';
+        nmz_strlower(_norc);
+
+        if (namazunorc[0] == '\0') {
+            set_namazunorc(_norc);
+        }
+    }
+
+    nmz_debug_printf("NAMAZUNORC: '%s'", _norc);
+    nmz_debug_printf("namazunorc: '%s'", namazunorc);
+
     /*
      *  1. $(sysconfdir)/$(PACKAGE)/namazurc
      *     - This can be overriden by environmentl variable NAMAZURC.
      */
-    {
+    if (strcmp("all", namazunorc) != 0 && strcmp("sysconfig", namazunorc) != 0) {
 	char *env = getenv_namazurc();
 	if (env != NULL) {
 	    if (load_rcfile(env) != SUCCESS) {
 		return FAILURE;
 	    }
 	} else {
-	    char fname[BUFSIZE];
+	    char fname[BUFSIZE] = "";
 	    strncpy(fname, namazurcdir, BUFSIZE - 1);
 	    strncat(fname, "/namazurc",
 		    BUFSIZE - strlen(fname) - 1);
@@ -755,10 +847,10 @@ load_rcfiles(void)
     /*
      *  2. ~/.namazurc
      */
-    {
+    if (strcmp("all", namazunorc) != 0 && strcmp("home", namazunorc) != 0) {
 	char *home = getenv("HOME");
 	if (home != NULL) {
-	    char fname[BUFSIZE];
+	    char fname[BUFSIZE] = "";
 	    strncpy(fname, home, BUFSIZE - 1);
 	    strncat(fname, "/.namazurc",
 		    BUFSIZE - strlen(fname) - 1);
